# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import timestamp_pb2  # type: ignore
import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.cloud.osconfig.v1",
    manifest={
        "VulnerabilityReport",
        "GetVulnerabilityReportRequest",
        "ListVulnerabilityReportsRequest",
        "ListVulnerabilityReportsResponse",
        "CVSSv3",
    },
)


class VulnerabilityReport(proto.Message):
    r"""This API resource represents the vulnerability report for a
    specified Compute Engine virtual machine (VM) instance at a given
    point in time.

    For more information, see `Vulnerability
    reports <https://cloud.google.com/compute/docs/instances/os-inventory-management#vulnerability-reports>`__.

    Attributes:
        name (str):
            Output only. The ``vulnerabilityReport`` API resource name.

            Format:
            ``projects/{project_number}/locations/{location}/instances/{instance_id}/vulnerabilityReport``
        vulnerabilities (MutableSequence[google.cloud.osconfig_v1.types.VulnerabilityReport.Vulnerability]):
            Output only. List of vulnerabilities
            affecting the VM.
        update_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. The timestamp for when the last
            vulnerability report was generated for the VM.
    """

    class Vulnerability(proto.Message):
        r"""A vulnerability affecting the VM instance.

        Attributes:
            details (google.cloud.osconfig_v1.types.VulnerabilityReport.Vulnerability.Details):
                Contains metadata as per the upstream feed of
                the operating system and NVD.
            installed_inventory_item_ids (MutableSequence[str]):
                Corresponds to the ``INSTALLED_PACKAGE`` inventory item on
                the VM. This field displays the inventory items affected by
                this vulnerability. If the vulnerability report was not
                updated after the VM inventory update, these values might
                not display in VM inventory. For some distros, this field
                may be empty.
            available_inventory_item_ids (MutableSequence[str]):
                Corresponds to the ``AVAILABLE_PACKAGE`` inventory item on
                the VM. If the vulnerability report was not updated after
                the VM inventory update, these values might not display in
                VM inventory. If there is no available fix, the field is
                empty. The ``inventory_item`` value specifies the latest
                ``SoftwarePackage`` available to the VM that fixes the
                vulnerability.
            create_time (google.protobuf.timestamp_pb2.Timestamp):
                The timestamp for when the vulnerability was
                first detected.
            update_time (google.protobuf.timestamp_pb2.Timestamp):
                The timestamp for when the vulnerability was
                last modified.
            items (MutableSequence[google.cloud.osconfig_v1.types.VulnerabilityReport.Vulnerability.Item]):
                List of items affected by the vulnerability.
        """

        class Details(proto.Message):
            r"""Contains metadata information for the vulnerability. This
            information is collected from the upstream feed of the operating
            system.

            Attributes:
                cve (str):
                    The CVE of the vulnerability. CVE cannot be
                    empty and the combination of <cve,
                    classification> should be unique across
                    vulnerabilities for a VM.
                cvss_v2_score (float):
                    The CVSS V2 score of this vulnerability. CVSS
                    V2 score is on a scale of 0 - 10 where 0
                    indicates low severity and 10 indicates high
                    severity.
                cvss_v3 (google.cloud.osconfig_v1.types.CVSSv3):
                    The full description of the CVSSv3 for this
                    vulnerability from NVD.
                severity (str):
                    Assigned severity/impact ranking from the
                    distro.
                description (str):
                    The note or description describing the
                    vulnerability from the distro.
                references (MutableSequence[google.cloud.osconfig_v1.types.VulnerabilityReport.Vulnerability.Details.Reference]):
                    Corresponds to the references attached to the
                    ``VulnerabilityDetails``.
            """

            class Reference(proto.Message):
                r"""A reference for this vulnerability.

                Attributes:
                    url (str):
                        The url of the reference.
                    source (str):
                        The source of the reference e.g. NVD.
                """

                url: str = proto.Field(
                    proto.STRING,
                    number=1,
                )
                source: str = proto.Field(
                    proto.STRING,
                    number=2,
                )

            cve: str = proto.Field(
                proto.STRING,
                number=1,
            )
            cvss_v2_score: float = proto.Field(
                proto.FLOAT,
                number=2,
            )
            cvss_v3: "CVSSv3" = proto.Field(
                proto.MESSAGE,
                number=3,
                message="CVSSv3",
            )
            severity: str = proto.Field(
                proto.STRING,
                number=4,
            )
            description: str = proto.Field(
                proto.STRING,
                number=5,
            )
            references: MutableSequence[
                "VulnerabilityReport.Vulnerability.Details.Reference"
            ] = proto.RepeatedField(
                proto.MESSAGE,
                number=6,
                message="VulnerabilityReport.Vulnerability.Details.Reference",
            )

        class Item(proto.Message):
            r"""OS inventory item that is affected by a vulnerability or
            fixed as a result of a vulnerability.

            Attributes:
                installed_inventory_item_id (str):
                    Corresponds to the ``INSTALLED_PACKAGE`` inventory item on
                    the VM. This field displays the inventory items affected by
                    this vulnerability. If the vulnerability report was not
                    updated after the VM inventory update, these values might
                    not display in VM inventory. For some operating systems,
                    this field might be empty.
                available_inventory_item_id (str):
                    Corresponds to the ``AVAILABLE_PACKAGE`` inventory item on
                    the VM. If the vulnerability report was not updated after
                    the VM inventory update, these values might not display in
                    VM inventory. If there is no available fix, the field is
                    empty. The ``inventory_item`` value specifies the latest
                    ``SoftwarePackage`` available to the VM that fixes the
                    vulnerability.
                fixed_cpe_uri (str):
                    The recommended `CPE
                    URI <https://cpe.mitre.org/specification/>`__ update that
                    contains a fix for this vulnerability.
                upstream_fix (str):
                    The upstream OS patch, packages or KB that
                    fixes the vulnerability.
            """

            installed_inventory_item_id: str = proto.Field(
                proto.STRING,
                number=1,
            )
            available_inventory_item_id: str = proto.Field(
                proto.STRING,
                number=2,
            )
            fixed_cpe_uri: str = proto.Field(
                proto.STRING,
                number=3,
            )
            upstream_fix: str = proto.Field(
                proto.STRING,
                number=4,
            )

        details: "VulnerabilityReport.Vulnerability.Details" = proto.Field(
            proto.MESSAGE,
            number=1,
            message="VulnerabilityReport.Vulnerability.Details",
        )
        installed_inventory_item_ids: MutableSequence[str] = proto.RepeatedField(
            proto.STRING,
            number=2,
        )
        available_inventory_item_ids: MutableSequence[str] = proto.RepeatedField(
            proto.STRING,
            number=3,
        )
        create_time: timestamp_pb2.Timestamp = proto.Field(
            proto.MESSAGE,
            number=4,
            message=timestamp_pb2.Timestamp,
        )
        update_time: timestamp_pb2.Timestamp = proto.Field(
            proto.MESSAGE,
            number=5,
            message=timestamp_pb2.Timestamp,
        )
        items: MutableSequence[
            "VulnerabilityReport.Vulnerability.Item"
        ] = proto.RepeatedField(
            proto.MESSAGE,
            number=6,
            message="VulnerabilityReport.Vulnerability.Item",
        )

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    vulnerabilities: MutableSequence[Vulnerability] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message=Vulnerability,
    )
    update_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )


class GetVulnerabilityReportRequest(proto.Message):
    r"""A request message for getting the vulnerability report for
    the specified VM.

    Attributes:
        name (str):
            Required. API resource name for vulnerability resource.

            Format:
            ``projects/{project}/locations/{location}/instances/{instance}/vulnerabilityReport``

            For ``{project}``, either ``project-number`` or
            ``project-id`` can be provided. For ``{instance}``, either
            Compute Engine ``instance-id`` or ``instance-name`` can be
            provided.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ListVulnerabilityReportsRequest(proto.Message):
    r"""A request message for listing vulnerability reports for all
    VM instances in the specified location.

    Attributes:
        parent (str):
            Required. The parent resource name.

            Format:
            ``projects/{project}/locations/{location}/instances/-``

            For ``{project}``, either ``project-number`` or
            ``project-id`` can be provided.
        page_size (int):
            The maximum number of results to return.
        page_token (str):
            A pagination token returned from a previous call to
            ``ListVulnerabilityReports`` that indicates where this
            listing should continue from.
        filter (str):
            If provided, this field specifies the criteria that must be
            met by a ``vulnerabilityReport`` API resource to be included
            in the response.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=2,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=3,
    )
    filter: str = proto.Field(
        proto.STRING,
        number=4,
    )


class ListVulnerabilityReportsResponse(proto.Message):
    r"""A response message for listing vulnerability reports for all
    VM instances in the specified location.

    Attributes:
        vulnerability_reports (MutableSequence[google.cloud.osconfig_v1.types.VulnerabilityReport]):
            List of vulnerabilityReport objects.
        next_page_token (str):
            The pagination token to retrieve the next
            page of vulnerabilityReports object.
    """

    @property
    def raw_page(self):
        return self

    vulnerability_reports: MutableSequence["VulnerabilityReport"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="VulnerabilityReport",
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class CVSSv3(proto.Message):
    r"""Common Vulnerability Scoring System version 3.
    For details, see
    https://www.first.org/cvss/specification-document

    Attributes:
        base_score (float):
            The base score is a function of the base
            metric scores.
            https://www.first.org/cvss/specification-document#Base-Metrics
        exploitability_score (float):
            The Exploitability sub-score equation is
            derived from the Base Exploitability metrics.
            https://www.first.org/cvss/specification-document#2-1-Exploitability-Metrics
        impact_score (float):
            The Impact sub-score equation is derived from
            the Base Impact metrics.
        attack_vector (google.cloud.osconfig_v1.types.CVSSv3.AttackVector):
            This metric reflects the context by which
            vulnerability exploitation is possible.
        attack_complexity (google.cloud.osconfig_v1.types.CVSSv3.AttackComplexity):
            This metric describes the conditions beyond
            the attacker's control that must exist in order
            to exploit the vulnerability.
        privileges_required (google.cloud.osconfig_v1.types.CVSSv3.PrivilegesRequired):
            This metric describes the level of privileges
            an attacker must possess before successfully
            exploiting the vulnerability.
        user_interaction (google.cloud.osconfig_v1.types.CVSSv3.UserInteraction):
            This metric captures the requirement for a
            human user, other than the attacker, to
            participate in the successful compromise of the
            vulnerable component.
        scope (google.cloud.osconfig_v1.types.CVSSv3.Scope):
            The Scope metric captures whether a
            vulnerability in one vulnerable component
            impacts resources in components beyond its
            security scope.
        confidentiality_impact (google.cloud.osconfig_v1.types.CVSSv3.Impact):
            This metric measures the impact to the
            confidentiality of the information resources
            managed by a software component due to a
            successfully exploited vulnerability.
        integrity_impact (google.cloud.osconfig_v1.types.CVSSv3.Impact):
            This metric measures the impact to integrity
            of a successfully exploited vulnerability.
        availability_impact (google.cloud.osconfig_v1.types.CVSSv3.Impact):
            This metric measures the impact to the
            availability of the impacted component resulting
            from a successfully exploited vulnerability.
    """

    class AttackVector(proto.Enum):
        r"""This metric reflects the context by which vulnerability
        exploitation is possible.

        Values:
            ATTACK_VECTOR_UNSPECIFIED (0):
                Invalid value.
            ATTACK_VECTOR_NETWORK (1):
                The vulnerable component is bound to the
                network stack and the set of possible attackers
                extends beyond the other options listed below,
                up to and including the entire Internet.
            ATTACK_VECTOR_ADJACENT (2):
                The vulnerable component is bound to the
                network stack, but the attack is limited at the
                protocol level to a logically adjacent topology.
            ATTACK_VECTOR_LOCAL (3):
                The vulnerable component is not bound to the
                network stack and the attacker's path is via
                read/write/execute capabilities.
            ATTACK_VECTOR_PHYSICAL (4):
                The attack requires the attacker to
                physically touch or manipulate the vulnerable
                component.
        """
        ATTACK_VECTOR_UNSPECIFIED = 0
        ATTACK_VECTOR_NETWORK = 1
        ATTACK_VECTOR_ADJACENT = 2
        ATTACK_VECTOR_LOCAL = 3
        ATTACK_VECTOR_PHYSICAL = 4

    class AttackComplexity(proto.Enum):
        r"""This metric describes the conditions beyond the attacker's
        control that must exist in order to exploit the vulnerability.

        Values:
            ATTACK_COMPLEXITY_UNSPECIFIED (0):
                Invalid value.
            ATTACK_COMPLEXITY_LOW (1):
                Specialized access conditions or extenuating
                circumstances do not exist. An attacker can
                expect repeatable success when attacking the
                vulnerable component.
            ATTACK_COMPLEXITY_HIGH (2):
                A successful attack depends on conditions
                beyond the attacker's control. That is, a
                successful attack cannot be accomplished at
                will, but requires the attacker to invest in
                some measurable amount of effort in preparation
                or execution against the vulnerable component
                before a successful attack can be expected.
        """
        ATTACK_COMPLEXITY_UNSPECIFIED = 0
        ATTACK_COMPLEXITY_LOW = 1
        ATTACK_COMPLEXITY_HIGH = 2

    class PrivilegesRequired(proto.Enum):
        r"""This metric describes the level of privileges an attacker
        must possess before successfully exploiting the vulnerability.

        Values:
            PRIVILEGES_REQUIRED_UNSPECIFIED (0):
                Invalid value.
            PRIVILEGES_REQUIRED_NONE (1):
                The attacker is unauthorized prior to attack,
                and therefore does not require any access to
                settings or files of the vulnerable system to
                carry out an attack.
            PRIVILEGES_REQUIRED_LOW (2):
                The attacker requires privileges that provide
                basic user capabilities that could normally
                affect only settings and files owned by a user.
                Alternatively, an attacker with Low privileges
                has the ability to access only non-sensitive
                resources.
            PRIVILEGES_REQUIRED_HIGH (3):
                The attacker requires privileges that provide
                significant (e.g., administrative) control over
                the vulnerable component allowing access to
                component-wide settings and files.
        """
        PRIVILEGES_REQUIRED_UNSPECIFIED = 0
        PRIVILEGES_REQUIRED_NONE = 1
        PRIVILEGES_REQUIRED_LOW = 2
        PRIVILEGES_REQUIRED_HIGH = 3

    class UserInteraction(proto.Enum):
        r"""This metric captures the requirement for a human user, other
        than the attacker, to participate in the successful compromise
        of the vulnerable component.

        Values:
            USER_INTERACTION_UNSPECIFIED (0):
                Invalid value.
            USER_INTERACTION_NONE (1):
                The vulnerable system can be exploited
                without interaction from any user.
            USER_INTERACTION_REQUIRED (2):
                Successful exploitation of this vulnerability
                requires a user to take some action before the
                vulnerability can be exploited.
        """
        USER_INTERACTION_UNSPECIFIED = 0
        USER_INTERACTION_NONE = 1
        USER_INTERACTION_REQUIRED = 2

    class Scope(proto.Enum):
        r"""The Scope metric captures whether a vulnerability in one
        vulnerable component impacts resources in components beyond its
        security scope.

        Values:
            SCOPE_UNSPECIFIED (0):
                Invalid value.
            SCOPE_UNCHANGED (1):
                An exploited vulnerability can only affect
                resources managed by the same security
                authority.
            SCOPE_CHANGED (2):
                An exploited vulnerability can affect
                resources beyond the security scope managed by
                the security authority of the vulnerable
                component.
        """
        SCOPE_UNSPECIFIED = 0
        SCOPE_UNCHANGED = 1
        SCOPE_CHANGED = 2

    class Impact(proto.Enum):
        r"""The Impact metrics capture the effects of a successfully
        exploited vulnerability on the component that suffers the worst
        outcome that is most directly and predictably associated with
        the attack.

        Values:
            IMPACT_UNSPECIFIED (0):
                Invalid value.
            IMPACT_HIGH (1):
                High impact.
            IMPACT_LOW (2):
                Low impact.
            IMPACT_NONE (3):
                No impact.
        """
        IMPACT_UNSPECIFIED = 0
        IMPACT_HIGH = 1
        IMPACT_LOW = 2
        IMPACT_NONE = 3

    base_score: float = proto.Field(
        proto.FLOAT,
        number=1,
    )
    exploitability_score: float = proto.Field(
        proto.FLOAT,
        number=2,
    )
    impact_score: float = proto.Field(
        proto.FLOAT,
        number=3,
    )
    attack_vector: AttackVector = proto.Field(
        proto.ENUM,
        number=5,
        enum=AttackVector,
    )
    attack_complexity: AttackComplexity = proto.Field(
        proto.ENUM,
        number=6,
        enum=AttackComplexity,
    )
    privileges_required: PrivilegesRequired = proto.Field(
        proto.ENUM,
        number=7,
        enum=PrivilegesRequired,
    )
    user_interaction: UserInteraction = proto.Field(
        proto.ENUM,
        number=8,
        enum=UserInteraction,
    )
    scope: Scope = proto.Field(
        proto.ENUM,
        number=9,
        enum=Scope,
    )
    confidentiality_impact: Impact = proto.Field(
        proto.ENUM,
        number=10,
        enum=Impact,
    )
    integrity_impact: Impact = proto.Field(
        proto.ENUM,
        number=11,
        enum=Impact,
    )
    availability_impact: Impact = proto.Field(
        proto.ENUM,
        number=12,
        enum=Impact,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import timestamp_pb2  # type: ignore
import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.cloud.osconfig.v1",
    manifest={
        "GetOSPolicyAssignmentReportRequest",
        "ListOSPolicyAssignmentReportsRequest",
        "ListOSPolicyAssignmentReportsResponse",
        "OSPolicyAssignmentReport",
    },
)


class GetOSPolicyAssignmentReportRequest(proto.Message):
    r"""Get a report of the OS policy assignment for a VM instance.

    Attributes:
        name (str):
            Required. API resource name for OS policy assignment report.

            Format:
            ``/projects/{project}/locations/{location}/instances/{instance}/osPolicyAssignments/{assignment}/report``

            For ``{project}``, either ``project-number`` or
            ``project-id`` can be provided. For ``{instance_id}``,
            either Compute Engine ``instance-id`` or ``instance-name``
            can be provided. For ``{assignment_id}``, the
            OSPolicyAssignment id must be provided.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ListOSPolicyAssignmentReportsRequest(proto.Message):
    r"""List the OS policy assignment reports for VM instances.

    Attributes:
        parent (str):
            Required. The parent resource name.

            Format:
            ``projects/{project}/locations/{location}/instances/{instance}/osPolicyAssignments/{assignment}/reports``

            For ``{project}``, either ``project-number`` or
            ``project-id`` can be provided. For ``{instance}``, either
            ``instance-name``, ``instance-id``, or ``-`` can be
            provided. If '-' is provided, the response will include
            OSPolicyAssignmentReports for all instances in the
            project/location. For ``{assignment}``, either
            ``assignment-id`` or ``-`` can be provided. If '-' is
            provided, the response will include
            OSPolicyAssignmentReports for all OSPolicyAssignments in the
            project/location. Either {instance} or {assignment} must be
            ``-``.

            For example:
            ``projects/{project}/locations/{location}/instances/{instance}/osPolicyAssignments/-/reports``
            returns all reports for the instance
            ``projects/{project}/locations/{location}/instances/-/osPolicyAssignments/{assignment-id}/reports``
            returns all the reports for the given assignment across all
            instances.
            ``projects/{project}/locations/{location}/instances/-/osPolicyAssignments/-/reports``
            returns all the reports for all assignments across all
            instances.
        page_size (int):
            The maximum number of results to return.
        filter (str):
            If provided, this field specifies the criteria that must be
            met by the ``OSPolicyAssignmentReport`` API resource that is
            included in the response.
        page_token (str):
            A pagination token returned from a previous call to the
            ``ListOSPolicyAssignmentReports`` method that indicates
            where this listing should continue from.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=2,
    )
    filter: str = proto.Field(
        proto.STRING,
        number=3,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=4,
    )


class ListOSPolicyAssignmentReportsResponse(proto.Message):
    r"""A response message for listing OS Policy assignment reports
    including the page of results and page token.

    Attributes:
        os_policy_assignment_reports (MutableSequence[google.cloud.osconfig_v1.types.OSPolicyAssignmentReport]):
            List of OS policy assignment reports.
        next_page_token (str):
            The pagination token to retrieve the next
            page of OS policy assignment report objects.
    """

    @property
    def raw_page(self):
        return self

    os_policy_assignment_reports: MutableSequence[
        "OSPolicyAssignmentReport"
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="OSPolicyAssignmentReport",
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class OSPolicyAssignmentReport(proto.Message):
    r"""A report of the OS policy assignment status for a given
    instance.

    Attributes:
        name (str):
            The ``OSPolicyAssignmentReport`` API resource name.

            Format:
            ``projects/{project_number}/locations/{location}/instances/{instance_id}/osPolicyAssignments/{os_policy_assignment_id}/report``
        instance (str):
            The Compute Engine VM instance name.
        os_policy_assignment (str):
            Reference to the ``OSPolicyAssignment`` API resource that
            the ``OSPolicy`` belongs to.

            Format:
            ``projects/{project_number}/locations/{location}/osPolicyAssignments/{os_policy_assignment_id@revision_id}``
        os_policy_compliances (MutableSequence[google.cloud.osconfig_v1.types.OSPolicyAssignmentReport.OSPolicyCompliance]):
            Compliance data for each ``OSPolicy`` that is applied to the
            VM.
        update_time (google.protobuf.timestamp_pb2.Timestamp):
            Timestamp for when the report was last
            generated.
        last_run_id (str):
            Unique identifier of the last attempted run
            to apply the OS policies associated with this
            assignment on the VM.
            This ID is logged by the OS Config agent while
            applying the OS policies associated with this
            assignment on the VM. NOTE: If the service is
            unable to successfully connect to the agent for
            this run, then this id will not be available in
            the agent logs.
    """

    class OSPolicyCompliance(proto.Message):
        r"""Compliance data for an OS policy

        Attributes:
            os_policy_id (str):
                The OS policy id
            compliance_state (google.cloud.osconfig_v1.types.OSPolicyAssignmentReport.OSPolicyCompliance.ComplianceState):
                The compliance state of the OS policy.
            compliance_state_reason (str):
                The reason for the OS policy to be in an unknown compliance
                state. This field is always populated when
                ``compliance_state`` is ``UNKNOWN``.

                If populated, the field can contain one of the following
                values:

                -  ``vm-not-running``: The VM was not running.
                -  ``os-policies-not-supported-by-agent``: The version of
                   the OS Config agent running on the VM does not support
                   running OS policies.
                -  ``no-agent-detected``: The OS Config agent is not
                   detected for the VM.
                -  ``resource-execution-errors``: The OS Config agent
                   encountered errors while executing one or more resources
                   in the policy. See ``os_policy_resource_compliances`` for
                   details.
                -  ``task-timeout``: The task sent to the agent to apply the
                   policy timed out.
                -  ``unexpected-agent-state``: The OS Config agent did not
                   report the final status of the task that attempted to
                   apply the policy. Instead, the agent unexpectedly started
                   working on a different task. This mostly happens when the
                   agent or VM unexpectedly restarts while applying OS
                   policies.
                -  ``internal-service-errors``: Internal service errors were
                   encountered while attempting to apply the policy.
            os_policy_resource_compliances (MutableSequence[google.cloud.osconfig_v1.types.OSPolicyAssignmentReport.OSPolicyCompliance.OSPolicyResourceCompliance]):
                Compliance data for each resource within the
                policy that is applied to the VM.
        """

        class ComplianceState(proto.Enum):
            r"""Possible compliance states for an os policy.

            Values:
                UNKNOWN (0):
                    The policy is in an unknown compliance state.

                    Refer to the field ``compliance_state_reason`` to learn the
                    exact reason for the policy to be in this compliance state.
                COMPLIANT (1):
                    Policy is compliant.
                    The policy is compliant if all the underlying
                    resources are also compliant.
                NON_COMPLIANT (2):
                    Policy is non-compliant.
                    The policy is non-compliant if one or more
                    underlying resources are non-compliant.
            """
            UNKNOWN = 0
            COMPLIANT = 1
            NON_COMPLIANT = 2

        class OSPolicyResourceCompliance(proto.Message):
            r"""Compliance data for an OS policy resource.

            .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

            Attributes:
                os_policy_resource_id (str):
                    The ID of the OS policy resource.
                config_steps (MutableSequence[google.cloud.osconfig_v1.types.OSPolicyAssignmentReport.OSPolicyCompliance.OSPolicyResourceCompliance.OSPolicyResourceConfigStep]):
                    Ordered list of configuration completed by
                    the agent for the OS policy resource.
                compliance_state (google.cloud.osconfig_v1.types.OSPolicyAssignmentReport.OSPolicyCompliance.OSPolicyResourceCompliance.ComplianceState):
                    The compliance state of the resource.
                compliance_state_reason (str):
                    A reason for the resource to be in the given compliance
                    state. This field is always populated when
                    ``compliance_state`` is ``UNKNOWN``.

                    The following values are supported when
                    ``compliance_state == UNKNOWN``

                    -  ``execution-errors``: Errors were encountered by the
                       agent while executing the resource and the compliance
                       state couldn't be determined.
                    -  ``execution-skipped-by-agent``: Resource execution was
                       skipped by the agent because errors were encountered
                       while executing prior resources in the OS policy.
                    -  ``os-policy-execution-attempt-failed``: The execution of
                       the OS policy containing this resource failed and the
                       compliance state couldn't be determined.
                exec_resource_output (google.cloud.osconfig_v1.types.OSPolicyAssignmentReport.OSPolicyCompliance.OSPolicyResourceCompliance.ExecResourceOutput):
                    ExecResource specific output.

                    This field is a member of `oneof`_ ``output``.
            """

            class ComplianceState(proto.Enum):
                r"""Possible compliance states for a resource.

                Values:
                    UNKNOWN (0):
                        The resource is in an unknown compliance state.

                        To get more details about why the policy is in this state,
                        review the output of the ``compliance_state_reason`` field.
                    COMPLIANT (1):
                        Resource is compliant.
                    NON_COMPLIANT (2):
                        Resource is non-compliant.
                """
                UNKNOWN = 0
                COMPLIANT = 1
                NON_COMPLIANT = 2

            class OSPolicyResourceConfigStep(proto.Message):
                r"""Step performed by the OS Config agent for configuring an
                ``OSPolicy`` resource to its desired state.

                Attributes:
                    type_ (google.cloud.osconfig_v1.types.OSPolicyAssignmentReport.OSPolicyCompliance.OSPolicyResourceCompliance.OSPolicyResourceConfigStep.Type):
                        Configuration step type.
                    error_message (str):
                        An error message recorded during the
                        execution of this step. Only populated if errors
                        were encountered during this step execution.
                """

                class Type(proto.Enum):
                    r"""Supported configuration step types

                    Values:
                        TYPE_UNSPECIFIED (0):
                            Default value. This value is unused.
                        VALIDATION (1):
                            Checks for resource conflicts such as schema
                            errors.
                        DESIRED_STATE_CHECK (2):
                            Checks the current status of the desired
                            state for a resource.
                        DESIRED_STATE_ENFORCEMENT (3):
                            Enforces the desired state for a resource
                            that is not in desired state.
                        DESIRED_STATE_CHECK_POST_ENFORCEMENT (4):
                            Re-checks the status of the desired state.
                            This check is done for a resource after the
                            enforcement of all OS policies.
                            This step is used to determine the final desired
                            state status for the resource. It accounts for
                            any resources that might have drifted from their
                            desired state due to side effects from executing
                            other resources.
                    """
                    TYPE_UNSPECIFIED = 0
                    VALIDATION = 1
                    DESIRED_STATE_CHECK = 2
                    DESIRED_STATE_ENFORCEMENT = 3
                    DESIRED_STATE_CHECK_POST_ENFORCEMENT = 4

                type_: "OSPolicyAssignmentReport.OSPolicyCompliance.OSPolicyResourceCompliance.OSPolicyResourceConfigStep.Type" = proto.Field(
                    proto.ENUM,
                    number=1,
                    enum="OSPolicyAssignmentReport.OSPolicyCompliance.OSPolicyResourceCompliance.OSPolicyResourceConfigStep.Type",
                )
                error_message: str = proto.Field(
                    proto.STRING,
                    number=2,
                )

            class ExecResourceOutput(proto.Message):
                r"""ExecResource specific output.

                Attributes:
                    enforcement_output (bytes):
                        Output from enforcement phase output file (if
                        run). Output size is limited to 100K bytes.
                """

                enforcement_output: bytes = proto.Field(
                    proto.BYTES,
                    number=2,
                )

            os_policy_resource_id: str = proto.Field(
                proto.STRING,
                number=1,
            )
            config_steps: MutableSequence[
                "OSPolicyAssignmentReport.OSPolicyCompliance.OSPolicyResourceCompliance.OSPolicyResourceConfigStep"
            ] = proto.RepeatedField(
                proto.MESSAGE,
                number=2,
                message="OSPolicyAssignmentReport.OSPolicyCompliance.OSPolicyResourceCompliance.OSPolicyResourceConfigStep",
            )
            compliance_state: "OSPolicyAssignmentReport.OSPolicyCompliance.OSPolicyResourceCompliance.ComplianceState" = proto.Field(
                proto.ENUM,
                number=3,
                enum="OSPolicyAssignmentReport.OSPolicyCompliance.OSPolicyResourceCompliance.ComplianceState",
            )
            compliance_state_reason: str = proto.Field(
                proto.STRING,
                number=4,
            )
            exec_resource_output: "OSPolicyAssignmentReport.OSPolicyCompliance.OSPolicyResourceCompliance.ExecResourceOutput" = proto.Field(
                proto.MESSAGE,
                number=5,
                oneof="output",
                message="OSPolicyAssignmentReport.OSPolicyCompliance.OSPolicyResourceCompliance.ExecResourceOutput",
            )

        os_policy_id: str = proto.Field(
            proto.STRING,
            number=1,
        )
        compliance_state: "OSPolicyAssignmentReport.OSPolicyCompliance.ComplianceState" = proto.Field(
            proto.ENUM,
            number=2,
            enum="OSPolicyAssignmentReport.OSPolicyCompliance.ComplianceState",
        )
        compliance_state_reason: str = proto.Field(
            proto.STRING,
            number=3,
        )
        os_policy_resource_compliances: MutableSequence[
            "OSPolicyAssignmentReport.OSPolicyCompliance.OSPolicyResourceCompliance"
        ] = proto.RepeatedField(
            proto.MESSAGE,
            number=4,
            message="OSPolicyAssignmentReport.OSPolicyCompliance.OSPolicyResourceCompliance",
        )

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    instance: str = proto.Field(
        proto.STRING,
        number=2,
    )
    os_policy_assignment: str = proto.Field(
        proto.STRING,
        number=3,
    )
    os_policy_compliances: MutableSequence[OSPolicyCompliance] = proto.RepeatedField(
        proto.MESSAGE,
        number=4,
        message=OSPolicyCompliance,
    )
    update_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=5,
        message=timestamp_pb2.Timestamp,
    )
    last_run_id: str = proto.Field(
        proto.STRING,
        number=6,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import duration_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore
import proto  # type: ignore

from google.cloud.osconfig_v1.types import osconfig_common

__protobuf__ = proto.module(
    package="google.cloud.osconfig.v1",
    manifest={
        "ExecutePatchJobRequest",
        "GetPatchJobRequest",
        "ListPatchJobInstanceDetailsRequest",
        "ListPatchJobInstanceDetailsResponse",
        "PatchJobInstanceDetails",
        "ListPatchJobsRequest",
        "ListPatchJobsResponse",
        "PatchJob",
        "PatchConfig",
        "Instance",
        "CancelPatchJobRequest",
        "AptSettings",
        "YumSettings",
        "GooSettings",
        "ZypperSettings",
        "WindowsUpdateSettings",
        "ExecStep",
        "ExecStepConfig",
        "GcsObject",
        "PatchInstanceFilter",
        "PatchRollout",
    },
)


class ExecutePatchJobRequest(proto.Message):
    r"""A request message to initiate patching across Compute Engine
    instances.

    Attributes:
        parent (str):
            Required. The project in which to run this patch in the form
            ``projects/*``
        description (str):
            Description of the patch job. Length of the
            description is limited to 1024 characters.
        instance_filter (google.cloud.osconfig_v1.types.PatchInstanceFilter):
            Required. Instances to patch, either
            explicitly or filtered by some criteria such as
            zone or labels.
        patch_config (google.cloud.osconfig_v1.types.PatchConfig):
            Patch configuration being applied. If
            omitted, instances are patched using the default
            configurations.
        duration (google.protobuf.duration_pb2.Duration):
            Duration of the patch job. After the duration
            ends, the patch job times out.
        dry_run (bool):
            If this patch is a dry-run only, instances
            are contacted but will do nothing.
        display_name (str):
            Display name for this patch job. This does
            not have to be unique.
        rollout (google.cloud.osconfig_v1.types.PatchRollout):
            Rollout strategy of the patch job.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    description: str = proto.Field(
        proto.STRING,
        number=2,
    )
    instance_filter: "PatchInstanceFilter" = proto.Field(
        proto.MESSAGE,
        number=7,
        message="PatchInstanceFilter",
    )
    patch_config: "PatchConfig" = proto.Field(
        proto.MESSAGE,
        number=4,
        message="PatchConfig",
    )
    duration: duration_pb2.Duration = proto.Field(
        proto.MESSAGE,
        number=5,
        message=duration_pb2.Duration,
    )
    dry_run: bool = proto.Field(
        proto.BOOL,
        number=6,
    )
    display_name: str = proto.Field(
        proto.STRING,
        number=8,
    )
    rollout: "PatchRollout" = proto.Field(
        proto.MESSAGE,
        number=9,
        message="PatchRollout",
    )


class GetPatchJobRequest(proto.Message):
    r"""Request to get an active or completed patch job.

    Attributes:
        name (str):
            Required. Name of the patch in the form
            ``projects/*/patchJobs/*``
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ListPatchJobInstanceDetailsRequest(proto.Message):
    r"""Request to list details for all instances that are part of a
    patch job.

    Attributes:
        parent (str):
            Required. The parent for the instances are in the form of
            ``projects/*/patchJobs/*``.
        page_size (int):
            The maximum number of instance details
            records to return.  Default is 100.
        page_token (str):
            A pagination token returned from a previous
            call that indicates where this listing should
            continue from.
        filter (str):
            A filter expression that filters results listed in the
            response. This field supports filtering results by instance
            zone, name, state, or ``failure_reason``.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=2,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=3,
    )
    filter: str = proto.Field(
        proto.STRING,
        number=4,
    )


class ListPatchJobInstanceDetailsResponse(proto.Message):
    r"""A response message for listing the instances details for a
    patch job.

    Attributes:
        patch_job_instance_details (MutableSequence[google.cloud.osconfig_v1.types.PatchJobInstanceDetails]):
            A list of instance status.
        next_page_token (str):
            A pagination token that can be used to get
            the next page of results.
    """

    @property
    def raw_page(self):
        return self

    patch_job_instance_details: MutableSequence[
        "PatchJobInstanceDetails"
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="PatchJobInstanceDetails",
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class PatchJobInstanceDetails(proto.Message):
    r"""Patch details for a VM instance. For more information about
    reviewing VM instance details, see `Listing all VM instance details
    for a specific patch
    job <https://cloud.google.com/compute/docs/os-patch-management/manage-patch-jobs#list-instance-details>`__.

    Attributes:
        name (str):
            The instance name in the form
            ``projects/*/zones/*/instances/*``
        instance_system_id (str):
            The unique identifier for the instance. This
            identifier is defined by the server.
        state (google.cloud.osconfig_v1.types.Instance.PatchState):
            Current state of instance patch.
        failure_reason (str):
            If the patch fails, this field provides the
            reason.
        attempt_count (int):
            The number of times the agent that the agent
            attempts to apply the patch.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    instance_system_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    state: "Instance.PatchState" = proto.Field(
        proto.ENUM,
        number=3,
        enum="Instance.PatchState",
    )
    failure_reason: str = proto.Field(
        proto.STRING,
        number=4,
    )
    attempt_count: int = proto.Field(
        proto.INT64,
        number=5,
    )


class ListPatchJobsRequest(proto.Message):
    r"""A request message for listing patch jobs.

    Attributes:
        parent (str):
            Required. In the form of ``projects/*``
        page_size (int):
            The maximum number of instance status to
            return.
        page_token (str):
            A pagination token returned from a previous
            call that indicates where this listing should
            continue from.
        filter (str):
            If provided, this field specifies the criteria that must be
            met by patch jobs to be included in the response. Currently,
            filtering is only available on the patch_deployment field.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=2,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=3,
    )
    filter: str = proto.Field(
        proto.STRING,
        number=4,
    )


class ListPatchJobsResponse(proto.Message):
    r"""A response message for listing patch jobs.

    Attributes:
        patch_jobs (MutableSequence[google.cloud.osconfig_v1.types.PatchJob]):
            The list of patch jobs.
        next_page_token (str):
            A pagination token that can be used to get
            the next page of results.
    """

    @property
    def raw_page(self):
        return self

    patch_jobs: MutableSequence["PatchJob"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="PatchJob",
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class PatchJob(proto.Message):
    r"""A high level representation of a patch job that is either in
    progress or has completed.

    Instance details are not included in the job. To paginate through
    instance details, use ListPatchJobInstanceDetails.

    For more information about patch jobs, see `Creating patch
    jobs <https://cloud.google.com/compute/docs/os-patch-management/create-patch-job>`__.

    Attributes:
        name (str):
            Unique identifier for this patch job in the form
            ``projects/*/patchJobs/*``
        display_name (str):
            Display name for this patch job. This is not
            a unique identifier.
        description (str):
            Description of the patch job. Length of the
            description is limited to 1024 characters.
        create_time (google.protobuf.timestamp_pb2.Timestamp):
            Time this patch job was created.
        update_time (google.protobuf.timestamp_pb2.Timestamp):
            Last time this patch job was updated.
        state (google.cloud.osconfig_v1.types.PatchJob.State):
            The current state of the PatchJob.
        instance_filter (google.cloud.osconfig_v1.types.PatchInstanceFilter):
            Instances to patch.
        patch_config (google.cloud.osconfig_v1.types.PatchConfig):
            Patch configuration being applied.
        duration (google.protobuf.duration_pb2.Duration):
            Duration of the patch job. After the duration
            ends, the patch job times out.
        instance_details_summary (google.cloud.osconfig_v1.types.PatchJob.InstanceDetailsSummary):
            Summary of instance details.
        dry_run (bool):
            If this patch job is a dry run, the agent
            reports that it has finished without running any
            updates on the VM instance.
        error_message (str):
            If this patch job failed, this message
            provides information about the failure.
        percent_complete (float):
            Reflects the overall progress of the patch
            job in the range of 0.0 being no progress to
            100.0 being complete.
        patch_deployment (str):
            Output only. Name of the patch deployment
            that created this patch job.
        rollout (google.cloud.osconfig_v1.types.PatchRollout):
            Rollout strategy being applied.
    """

    class State(proto.Enum):
        r"""Enumeration of the various states a patch job passes through
        as it executes.

        Values:
            STATE_UNSPECIFIED (0):
                State must be specified.
            STARTED (1):
                The patch job was successfully initiated.
            INSTANCE_LOOKUP (2):
                The patch job is looking up instances to run
                the patch on.
            PATCHING (3):
                Instances are being patched.
            SUCCEEDED (4):
                Patch job completed successfully.
            COMPLETED_WITH_ERRORS (5):
                Patch job completed but there were errors.
            CANCELED (6):
                The patch job was canceled.
            TIMED_OUT (7):
                The patch job timed out.
        """
        STATE_UNSPECIFIED = 0
        STARTED = 1
        INSTANCE_LOOKUP = 2
        PATCHING = 3
        SUCCEEDED = 4
        COMPLETED_WITH_ERRORS = 5
        CANCELED = 6
        TIMED_OUT = 7

    class InstanceDetailsSummary(proto.Message):
        r"""A summary of the current patch state across all instances that this
        patch job affects. Contains counts of instances in different states.
        These states map to ``InstancePatchState``. List patch job instance
        details to see the specific states of each instance.

        Attributes:
            pending_instance_count (int):
                Number of instances pending patch job.
            inactive_instance_count (int):
                Number of instances that are inactive.
            notified_instance_count (int):
                Number of instances notified about patch job.
            started_instance_count (int):
                Number of instances that have started.
            downloading_patches_instance_count (int):
                Number of instances that are downloading
                patches.
            applying_patches_instance_count (int):
                Number of instances that are applying
                patches.
            rebooting_instance_count (int):
                Number of instances rebooting.
            succeeded_instance_count (int):
                Number of instances that have completed
                successfully.
            succeeded_reboot_required_instance_count (int):
                Number of instances that require reboot.
            failed_instance_count (int):
                Number of instances that failed.
            acked_instance_count (int):
                Number of instances that have acked and will
                start shortly.
            timed_out_instance_count (int):
                Number of instances that exceeded the time
                out while applying the patch.
            pre_patch_step_instance_count (int):
                Number of instances that are running the
                pre-patch step.
            post_patch_step_instance_count (int):
                Number of instances that are running the
                post-patch step.
            no_agent_detected_instance_count (int):
                Number of instances that do not appear to be
                running the agent. Check to ensure that the
                agent is installed, running, and able to
                communicate with the service.
        """

        pending_instance_count: int = proto.Field(
            proto.INT64,
            number=1,
        )
        inactive_instance_count: int = proto.Field(
            proto.INT64,
            number=2,
        )
        notified_instance_count: int = proto.Field(
            proto.INT64,
            number=3,
        )
        started_instance_count: int = proto.Field(
            proto.INT64,
            number=4,
        )
        downloading_patches_instance_count: int = proto.Field(
            proto.INT64,
            number=5,
        )
        applying_patches_instance_count: int = proto.Field(
            proto.INT64,
            number=6,
        )
        rebooting_instance_count: int = proto.Field(
            proto.INT64,
            number=7,
        )
        succeeded_instance_count: int = proto.Field(
            proto.INT64,
            number=8,
        )
        succeeded_reboot_required_instance_count: int = proto.Field(
            proto.INT64,
            number=9,
        )
        failed_instance_count: int = proto.Field(
            proto.INT64,
            number=10,
        )
        acked_instance_count: int = proto.Field(
            proto.INT64,
            number=11,
        )
        timed_out_instance_count: int = proto.Field(
            proto.INT64,
            number=12,
        )
        pre_patch_step_instance_count: int = proto.Field(
            proto.INT64,
            number=13,
        )
        post_patch_step_instance_count: int = proto.Field(
            proto.INT64,
            number=14,
        )
        no_agent_detected_instance_count: int = proto.Field(
            proto.INT64,
            number=15,
        )

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    display_name: str = proto.Field(
        proto.STRING,
        number=14,
    )
    description: str = proto.Field(
        proto.STRING,
        number=2,
    )
    create_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )
    update_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=4,
        message=timestamp_pb2.Timestamp,
    )
    state: State = proto.Field(
        proto.ENUM,
        number=5,
        enum=State,
    )
    instance_filter: "PatchInstanceFilter" = proto.Field(
        proto.MESSAGE,
        number=13,
        message="PatchInstanceFilter",
    )
    patch_config: "PatchConfig" = proto.Field(
        proto.MESSAGE,
        number=7,
        message="PatchConfig",
    )
    duration: duration_pb2.Duration = proto.Field(
        proto.MESSAGE,
        number=8,
        message=duration_pb2.Duration,
    )
    instance_details_summary: InstanceDetailsSummary = proto.Field(
        proto.MESSAGE,
        number=9,
        message=InstanceDetailsSummary,
    )
    dry_run: bool = proto.Field(
        proto.BOOL,
        number=10,
    )
    error_message: str = proto.Field(
        proto.STRING,
        number=11,
    )
    percent_complete: float = proto.Field(
        proto.DOUBLE,
        number=12,
    )
    patch_deployment: str = proto.Field(
        proto.STRING,
        number=15,
    )
    rollout: "PatchRollout" = proto.Field(
        proto.MESSAGE,
        number=16,
        message="PatchRollout",
    )


class PatchConfig(proto.Message):
    r"""Patch configuration specifications. Contains details on how
    to apply the patch(es) to a VM instance.

    Attributes:
        reboot_config (google.cloud.osconfig_v1.types.PatchConfig.RebootConfig):
            Post-patch reboot settings.
        apt (google.cloud.osconfig_v1.types.AptSettings):
            Apt update settings. Use this setting to override the
            default ``apt`` patch rules.
        yum (google.cloud.osconfig_v1.types.YumSettings):
            Yum update settings. Use this setting to override the
            default ``yum`` patch rules.
        goo (google.cloud.osconfig_v1.types.GooSettings):
            Goo update settings. Use this setting to override the
            default ``goo`` patch rules.
        zypper (google.cloud.osconfig_v1.types.ZypperSettings):
            Zypper update settings. Use this setting to override the
            default ``zypper`` patch rules.
        windows_update (google.cloud.osconfig_v1.types.WindowsUpdateSettings):
            Windows update settings. Use this override
            the default windows patch rules.
        pre_step (google.cloud.osconfig_v1.types.ExecStep):
            The ``ExecStep`` to run before the patch update.
        post_step (google.cloud.osconfig_v1.types.ExecStep):
            The ``ExecStep`` to run after the patch update.
        mig_instances_allowed (bool):
            Allows the patch job to run on Managed
            instance groups (MIGs).
    """

    class RebootConfig(proto.Enum):
        r"""Post-patch reboot settings.

        Values:
            REBOOT_CONFIG_UNSPECIFIED (0):
                The default behavior is DEFAULT.
            DEFAULT (1):
                The agent decides if a reboot is necessary by checking
                signals such as registry keys on Windows or
                ``/var/run/reboot-required`` on APT based systems. On RPM
                based systems, a set of core system package install times
                are compared with system boot time.
            ALWAYS (2):
                Always reboot the machine after the update
                completes.
            NEVER (3):
                Never reboot the machine after the update
                completes.
        """
        REBOOT_CONFIG_UNSPECIFIED = 0
        DEFAULT = 1
        ALWAYS = 2
        NEVER = 3

    reboot_config: RebootConfig = proto.Field(
        proto.ENUM,
        number=1,
        enum=RebootConfig,
    )
    apt: "AptSettings" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="AptSettings",
    )
    yum: "YumSettings" = proto.Field(
        proto.MESSAGE,
        number=4,
        message="YumSettings",
    )
    goo: "GooSettings" = proto.Field(
        proto.MESSAGE,
        number=5,
        message="GooSettings",
    )
    zypper: "ZypperSettings" = proto.Field(
        proto.MESSAGE,
        number=6,
        message="ZypperSettings",
    )
    windows_update: "WindowsUpdateSettings" = proto.Field(
        proto.MESSAGE,
        number=7,
        message="WindowsUpdateSettings",
    )
    pre_step: "ExecStep" = proto.Field(
        proto.MESSAGE,
        number=8,
        message="ExecStep",
    )
    post_step: "ExecStep" = proto.Field(
        proto.MESSAGE,
        number=9,
        message="ExecStep",
    )
    mig_instances_allowed: bool = proto.Field(
        proto.BOOL,
        number=10,
    )


class Instance(proto.Message):
    r"""Namespace for instance state enums."""

    class PatchState(proto.Enum):
        r"""Patch state of an instance.

        Values:
            PATCH_STATE_UNSPECIFIED (0):
                Unspecified.
            PENDING (1):
                The instance is not yet notified.
            INACTIVE (2):
                Instance is inactive and cannot be patched.
            NOTIFIED (3):
                The instance is notified that it should be
                patched.
            STARTED (4):
                The instance has started the patching
                process.
            DOWNLOADING_PATCHES (5):
                The instance is downloading patches.
            APPLYING_PATCHES (6):
                The instance is applying patches.
            REBOOTING (7):
                The instance is rebooting.
            SUCCEEDED (8):
                The instance has completed applying patches.
            SUCCEEDED_REBOOT_REQUIRED (9):
                The instance has completed applying patches
                but a reboot is required.
            FAILED (10):
                The instance has failed to apply the patch.
            ACKED (11):
                The instance acked the notification and will
                start shortly.
            TIMED_OUT (12):
                The instance exceeded the time out while
                applying the patch.
            RUNNING_PRE_PATCH_STEP (13):
                The instance is running the pre-patch step.
            RUNNING_POST_PATCH_STEP (14):
                The instance is running the post-patch step.
            NO_AGENT_DETECTED (15):
                The service could not detect the presence of
                the agent. Check to ensure that the agent is
                installed, running, and able to communicate with
                the service.
        """
        PATCH_STATE_UNSPECIFIED = 0
        PENDING = 1
        INACTIVE = 2
        NOTIFIED = 3
        STARTED = 4
        DOWNLOADING_PATCHES = 5
        APPLYING_PATCHES = 6
        REBOOTING = 7
        SUCCEEDED = 8
        SUCCEEDED_REBOOT_REQUIRED = 9
        FAILED = 10
        ACKED = 11
        TIMED_OUT = 12
        RUNNING_PRE_PATCH_STEP = 13
        RUNNING_POST_PATCH_STEP = 14
        NO_AGENT_DETECTED = 15


class CancelPatchJobRequest(proto.Message):
    r"""Message for canceling a patch job.

    Attributes:
        name (str):
            Required. Name of the patch in the form
            ``projects/*/patchJobs/*``
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class AptSettings(proto.Message):
    r"""Apt patching is completed by executing
    ``apt-get update && apt-get upgrade``. Additional options can be set
    to control how this is executed.

    Attributes:
        type_ (google.cloud.osconfig_v1.types.AptSettings.Type):
            By changing the type to DIST, the patching is performed
            using ``apt-get dist-upgrade`` instead.
        excludes (MutableSequence[str]):
            List of packages to exclude from update.
            These packages will be excluded
        exclusive_packages (MutableSequence[str]):
            An exclusive list of packages to be updated.
            These are the only packages that will be
            updated. If these packages are not installed,
            they will be ignored. This field cannot be
            specified with any other patch configuration
            fields.
    """

    class Type(proto.Enum):
        r"""Apt patch type.

        Values:
            TYPE_UNSPECIFIED (0):
                By default, upgrade will be performed.
            DIST (1):
                Runs ``apt-get dist-upgrade``.
            UPGRADE (2):
                Runs ``apt-get upgrade``.
        """
        TYPE_UNSPECIFIED = 0
        DIST = 1
        UPGRADE = 2

    type_: Type = proto.Field(
        proto.ENUM,
        number=1,
        enum=Type,
    )
    excludes: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=2,
    )
    exclusive_packages: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=3,
    )


class YumSettings(proto.Message):
    r"""Yum patching is performed by executing ``yum update``. Additional
    options can be set to control how this is executed.

    Note that not all settings are supported on all platforms.

    Attributes:
        security (bool):
            Adds the ``--security`` flag to ``yum update``. Not
            supported on all platforms.
        minimal (bool):
            Will cause patch to run ``yum update-minimal`` instead.
        excludes (MutableSequence[str]):
            List of packages to exclude from update. These packages are
            excluded by using the yum ``--exclude`` flag.
        exclusive_packages (MutableSequence[str]):
            An exclusive list of packages to be updated.
            These are the only packages that will be
            updated. If these packages are not installed,
            they will be ignored. This field must not be
            specified with any other patch configuration
            fields.
    """

    security: bool = proto.Field(
        proto.BOOL,
        number=1,
    )
    minimal: bool = proto.Field(
        proto.BOOL,
        number=2,
    )
    excludes: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=3,
    )
    exclusive_packages: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=4,
    )


class GooSettings(proto.Message):
    r"""Googet patching is performed by running ``googet update``."""


class ZypperSettings(proto.Message):
    r"""Zypper patching is performed by running ``zypper patch``. See also
    https://en.opensuse.org/SDB:Zypper_manual.

    Attributes:
        with_optional (bool):
            Adds the ``--with-optional`` flag to ``zypper patch``.
        with_update (bool):
            Adds the ``--with-update`` flag, to ``zypper patch``.
        categories (MutableSequence[str]):
            Install only patches with these categories.
            Common categories include security, recommended,
            and feature.
        severities (MutableSequence[str]):
            Install only patches with these severities.
            Common severities include critical, important,
            moderate, and low.
        excludes (MutableSequence[str]):
            List of patches to exclude from update.
        exclusive_patches (MutableSequence[str]):
            An exclusive list of patches to be updated. These are the
            only patches that will be installed using 'zypper patch
            patch:<patch_name>' command. This field must not be used
            with any other patch configuration fields.
    """

    with_optional: bool = proto.Field(
        proto.BOOL,
        number=1,
    )
    with_update: bool = proto.Field(
        proto.BOOL,
        number=2,
    )
    categories: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=3,
    )
    severities: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=4,
    )
    excludes: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=5,
    )
    exclusive_patches: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=6,
    )


class WindowsUpdateSettings(proto.Message):
    r"""Windows patching is performed using the Windows Update Agent.

    Attributes:
        classifications (MutableSequence[google.cloud.osconfig_v1.types.WindowsUpdateSettings.Classification]):
            Only apply updates of these windows update
            classifications. If empty, all updates are
            applied.
        excludes (MutableSequence[str]):
            List of KBs to exclude from update.
        exclusive_patches (MutableSequence[str]):
            An exclusive list of kbs to be updated. These
            are the only patches that will be updated. This
            field must not be used with other patch
            configurations.
    """

    class Classification(proto.Enum):
        r"""Microsoft Windows update classifications as defined in [1]
        https://support.microsoft.com/en-us/help/824684/description-of-the-standard-terminology-that-is-used-to-describe-micro

        Values:
            CLASSIFICATION_UNSPECIFIED (0):
                Invalid. If classifications are included,
                they must be specified.
            CRITICAL (1):
                "A widely released fix for a specific problem that addresses
                a critical, non-security-related bug." [1]
            SECURITY (2):
                "A widely released fix for a product-specific,
                security-related vulnerability. Security vulnerabilities are
                rated by their severity. The severity rating is indicated in
                the Microsoft security bulletin as critical, important,
                moderate, or low." [1]
            DEFINITION (3):
                "A widely released and frequent software update that
                contains additions to a product's definition database.
                Definition databases are often used to detect objects that
                have specific attributes, such as malicious code, phishing
                websites, or junk mail." [1]
            DRIVER (4):
                "Software that controls the input and output of a device."
                [1]
            FEATURE_PACK (5):
                "New product functionality that is first distributed outside
                the context of a product release and that is typically
                included in the next full product release." [1]
            SERVICE_PACK (6):
                "A tested, cumulative set of all hotfixes, security updates,
                critical updates, and updates. Additionally, service packs
                may contain additional fixes for problems that are found
                internally since the release of the product. Service packs
                my also contain a limited number of customer-requested
                design changes or features." [1]
            TOOL (7):
                "A utility or feature that helps complete a task or set of
                tasks." [1]
            UPDATE_ROLLUP (8):
                "A tested, cumulative set of hotfixes, security updates,
                critical updates, and updates that are packaged together for
                easy deployment. A rollup generally targets a specific area,
                such as security, or a component of a product, such as
                Internet Information Services (IIS)." [1]
            UPDATE (9):
                "A widely released fix for a specific problem. An update
                addresses a noncritical, non-security-related bug." [1]
        """
        CLASSIFICATION_UNSPECIFIED = 0
        CRITICAL = 1
        SECURITY = 2
        DEFINITION = 3
        DRIVER = 4
        FEATURE_PACK = 5
        SERVICE_PACK = 6
        TOOL = 7
        UPDATE_ROLLUP = 8
        UPDATE = 9

    classifications: MutableSequence[Classification] = proto.RepeatedField(
        proto.ENUM,
        number=1,
        enum=Classification,
    )
    excludes: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=2,
    )
    exclusive_patches: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=3,
    )


class ExecStep(proto.Message):
    r"""A step that runs an executable for a PatchJob.

    Attributes:
        linux_exec_step_config (google.cloud.osconfig_v1.types.ExecStepConfig):
            The ExecStepConfig for all Linux VMs targeted
            by the PatchJob.
        windows_exec_step_config (google.cloud.osconfig_v1.types.ExecStepConfig):
            The ExecStepConfig for all Windows VMs
            targeted by the PatchJob.
    """

    linux_exec_step_config: "ExecStepConfig" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="ExecStepConfig",
    )
    windows_exec_step_config: "ExecStepConfig" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="ExecStepConfig",
    )


class ExecStepConfig(proto.Message):
    r"""Common configurations for an ExecStep.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        local_path (str):
            An absolute path to the executable on the VM.

            This field is a member of `oneof`_ ``executable``.
        gcs_object (google.cloud.osconfig_v1.types.GcsObject):
            A Cloud Storage object containing the
            executable.

            This field is a member of `oneof`_ ``executable``.
        allowed_success_codes (MutableSequence[int]):
            Defaults to [0]. A list of possible return values that the
            execution can return to indicate a success.
        interpreter (google.cloud.osconfig_v1.types.ExecStepConfig.Interpreter):
            The script interpreter to use to run the script. If no
            interpreter is specified the script will be executed
            directly, which will likely only succeed for scripts with
            [shebang lines]
            (https://en.wikipedia.org/wiki/Shebang_(Unix)).
    """

    class Interpreter(proto.Enum):
        r"""The interpreter used to execute the a file.

        Values:
            INTERPRETER_UNSPECIFIED (0):
                Invalid for a Windows ExecStepConfig. For a
                Linux ExecStepConfig, the interpreter will be
                parsed from the shebang line of the script if
                unspecified.
            SHELL (1):
                Indicates that the script is run with ``/bin/sh`` on Linux
                and ``cmd`` on Windows.
            POWERSHELL (2):
                Indicates that the file is run with PowerShell flags
                ``-NonInteractive``, ``-NoProfile``, and
                ``-ExecutionPolicy Bypass``.
        """
        INTERPRETER_UNSPECIFIED = 0
        SHELL = 1
        POWERSHELL = 2

    local_path: str = proto.Field(
        proto.STRING,
        number=1,
        oneof="executable",
    )
    gcs_object: "GcsObject" = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="executable",
        message="GcsObject",
    )
    allowed_success_codes: MutableSequence[int] = proto.RepeatedField(
        proto.INT32,
        number=3,
    )
    interpreter: Interpreter = proto.Field(
        proto.ENUM,
        number=4,
        enum=Interpreter,
    )


class GcsObject(proto.Message):
    r"""Cloud Storage object representation.

    Attributes:
        bucket (str):
            Required. Bucket of the Cloud Storage object.
        object_ (str):
            Required. Name of the Cloud Storage object.
        generation_number (int):
            Required. Generation number of the Cloud
            Storage object. This is used to ensure that the
            ExecStep specified by this PatchJob does not
            change.
    """

    bucket: str = proto.Field(
        proto.STRING,
        number=1,
    )
    object_: str = proto.Field(
        proto.STRING,
        number=2,
    )
    generation_number: int = proto.Field(
        proto.INT64,
        number=3,
    )


class PatchInstanceFilter(proto.Message):
    r"""A filter to target VM instances for patching. The targeted
    VMs must meet all criteria specified. So if both labels and
    zones are specified, the patch job targets only VMs with those
    labels and in those zones.

    Attributes:
        all_ (bool):
            Target all VM instances in the project. If
            true, no other criteria is permitted.
        group_labels (MutableSequence[google.cloud.osconfig_v1.types.PatchInstanceFilter.GroupLabel]):
            Targets VM instances matching ANY of these
            GroupLabels. This allows targeting of disparate
            groups of VM instances.
        zones (MutableSequence[str]):
            Targets VM instances in ANY of these zones.
            Leave empty to target VM instances in any zone.
        instances (MutableSequence[str]):
            Targets any of the VM instances specified. Instances are
            specified by their URI in the form
            ``zones/[ZONE]/instances/[INSTANCE_NAME]``,
            ``projects/[PROJECT_ID]/zones/[ZONE]/instances/[INSTANCE_NAME]``,
            or
            ``https://www.googleapis.com/compute/v1/projects/[PROJECT_ID]/zones/[ZONE]/instances/[INSTANCE_NAME]``
        instance_name_prefixes (MutableSequence[str]):
            Targets VMs whose name starts with one of
            these prefixes. Similar to labels, this is
            another way to group VMs when targeting configs,
            for example prefix="prod-".
    """

    class GroupLabel(proto.Message):
        r"""Targets a group of VM instances by using their `assigned
        labels <https://cloud.google.com/compute/docs/labeling-resources>`__.
        Labels are key-value pairs. A ``GroupLabel`` is a combination of
        labels that is used to target VMs for a patch job.

        For example, a patch job can target VMs that have the following
        ``GroupLabel``: ``{"env":"test", "app":"web"}``. This means that the
        patch job is applied to VMs that have both the labels ``env=test``
        and ``app=web``.

        Attributes:
            labels (MutableMapping[str, str]):
                Compute Engine instance labels that must be
                present for a VM instance to be targeted by this
                filter.
        """

        labels: MutableMapping[str, str] = proto.MapField(
            proto.STRING,
            proto.STRING,
            number=1,
        )

    all_: bool = proto.Field(
        proto.BOOL,
        number=1,
    )
    group_labels: MutableSequence[GroupLabel] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message=GroupLabel,
    )
    zones: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=3,
    )
    instances: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=4,
    )
    instance_name_prefixes: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=5,
    )


class PatchRollout(proto.Message):
    r"""Patch rollout configuration specifications. Contains details
    on the concurrency control when applying patch(es) to all
    targeted VMs.

    Attributes:
        mode (google.cloud.osconfig_v1.types.PatchRollout.Mode):
            Mode of the patch rollout.
        disruption_budget (google.cloud.osconfig_v1.types.FixedOrPercent):
            The maximum number (or percentage) of VMs per zone to
            disrupt at any given moment. The number of VMs calculated
            from multiplying the percentage by the total number of VMs
            in a zone is rounded up.

            During patching, a VM is considered disrupted from the time
            the agent is notified to begin until patching has completed.
            This disruption time includes the time to complete reboot
            and any post-patch steps.

            A VM contributes to the disruption budget if its patching
            operation fails either when applying the patches, running
            pre or post patch steps, or if it fails to respond with a
            success notification before timing out. VMs that are not
            running or do not have an active agent do not count toward
            this disruption budget.

            For zone-by-zone rollouts, if the disruption budget in a
            zone is exceeded, the patch job stops, because continuing to
            the next zone requires completion of the patch process in
            the previous zone.

            For example, if the disruption budget has a fixed value of
            ``10``, and 8 VMs fail to patch in the current zone, the
            patch job continues to patch 2 VMs at a time until the zone
            is completed. When that zone is completed successfully,
            patching begins with 10 VMs at a time in the next zone. If
            10 VMs in the next zone fail to patch, the patch job stops.
    """

    class Mode(proto.Enum):
        r"""Type of the rollout.

        Values:
            MODE_UNSPECIFIED (0):
                Mode must be specified.
            ZONE_BY_ZONE (1):
                Patches are applied one zone at a time. The
                patch job begins in the region with the lowest
                number of targeted VMs. Within the region,
                patching begins in the zone with the lowest
                number of targeted VMs. If multiple regions (or
                zones within a region) have the same number of
                targeted VMs, a tie-breaker is achieved by
                sorting the regions or zones in alphabetical
                order.
            CONCURRENT_ZONES (2):
                Patches are applied to VMs in all zones at
                the same time.
        """
        MODE_UNSPECIFIED = 0
        ZONE_BY_ZONE = 1
        CONCURRENT_ZONES = 2

    mode: Mode = proto.Field(
        proto.ENUM,
        number=1,
        enum=Mode,
    )
    disruption_budget: osconfig_common.FixedOrPercent = proto.Field(
        proto.MESSAGE,
        number=2,
        message=osconfig_common.FixedOrPercent,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from typing import Awaitable, Callable, Dict, Optional, Sequence, Tuple, Union
import warnings

from google.api_core import gapic_v1, grpc_helpers_async, operations_v1
from google.auth import credentials as ga_credentials  # type: ignore
from google.auth.transport.grpc import SslCredentials  # type: ignore
from google.longrunning import operations_pb2  # type: ignore
import grpc  # type: ignore
from grpc.experimental import aio  # type: ignore

from google.cloud.osconfig_v1.types import (
    inventory,
    os_policy_assignment_reports,
    os_policy_assignments,
    vulnerability,
)

from .base import DEFAULT_CLIENT_INFO, OsConfigZonalServiceTransport
from .grpc import OsConfigZonalServiceGrpcTransport


class OsConfigZonalServiceGrpcAsyncIOTransport(OsConfigZonalServiceTransport):
    """gRPC AsyncIO backend transport for OsConfigZonalService.

    Zonal OS Config API
    The OS Config service is the server-side component that allows
    users to manage package installations and patch jobs for Compute
    Engine VM instances.

    This class defines the same methods as the primary client, so the
    primary client can load the underlying transport implementation
    and call it.

    It sends protocol buffers over the wire using gRPC (which is built on
    top of HTTP/2); the ``grpcio`` package must be installed.
    """

    _grpc_channel: aio.Channel
    _stubs: Dict[str, Callable] = {}

    @classmethod
    def create_channel(
        cls,
        host: str = "osconfig.googleapis.com",
        credentials: Optional[ga_credentials.Credentials] = None,
        credentials_file: Optional[str] = None,
        scopes: Optional[Sequence[str]] = None,
        quota_project_id: Optional[str] = None,
        **kwargs,
    ) -> aio.Channel:
        """Create and return a gRPC AsyncIO channel object.
        Args:
            host (Optional[str]): The host for the channel to use.
            credentials (Optional[~.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify this application to the service. If
                none are specified, the client will attempt to ascertain
                the credentials from the environment.
            credentials_file (Optional[str]): A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is ignored if ``channel`` is provided.
            scopes (Optional[Sequence[str]]): A optional list of scopes needed for this
                service. These are only used when credentials are not specified and
                are passed to :func:`google.auth.default`.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            kwargs (Optional[dict]): Keyword arguments, which are passed to the
                channel creation.
        Returns:
            aio.Channel: A gRPC AsyncIO channel object.
        """

        return grpc_helpers_async.create_channel(
            host,
            credentials=credentials,
            credentials_file=credentials_file,
            quota_project_id=quota_project_id,
            default_scopes=cls.AUTH_SCOPES,
            scopes=scopes,
            default_host=cls.DEFAULT_HOST,
            **kwargs,
        )

    def __init__(
        self,
        *,
        host: str = "osconfig.googleapis.com",
        credentials: Optional[ga_credentials.Credentials] = None,
        credentials_file: Optional[str] = None,
        scopes: Optional[Sequence[str]] = None,
        channel: Optional[aio.Channel] = None,
        api_mtls_endpoint: Optional[str] = None,
        client_cert_source: Optional[Callable[[], Tuple[bytes, bytes]]] = None,
        ssl_channel_credentials: Optional[grpc.ChannelCredentials] = None,
        client_cert_source_for_mtls: Optional[Callable[[], Tuple[bytes, bytes]]] = None,
        quota_project_id: Optional[str] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
        always_use_jwt_access: Optional[bool] = False,
        api_audience: Optional[str] = None,
    ) -> None:
        """Instantiate the transport.

        Args:
            host (Optional[str]):
                 The hostname to connect to.
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
                This argument is ignored if ``channel`` is provided.
            credentials_file (Optional[str]): A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is ignored if ``channel`` is provided.
            scopes (Optional[Sequence[str]]): A optional list of scopes needed for this
                service. These are only used when credentials are not specified and
                are passed to :func:`google.auth.default`.
            channel (Optional[aio.Channel]): A ``Channel`` instance through
                which to make calls.
            api_mtls_endpoint (Optional[str]): Deprecated. The mutual TLS endpoint.
                If provided, it overrides the ``host`` argument and tries to create
                a mutual TLS channel with client SSL credentials from
                ``client_cert_source`` or application default SSL credentials.
            client_cert_source (Optional[Callable[[], Tuple[bytes, bytes]]]):
                Deprecated. A callback to provide client SSL certificate bytes and
                private key bytes, both in PEM format. It is ignored if
                ``api_mtls_endpoint`` is None.
            ssl_channel_credentials (grpc.ChannelCredentials): SSL credentials
                for the grpc channel. It is ignored if ``channel`` is provided.
            client_cert_source_for_mtls (Optional[Callable[[], Tuple[bytes, bytes]]]):
                A callback to provide client certificate bytes and private key bytes,
                both in PEM format. It is used to configure a mutual TLS channel. It is
                ignored if ``channel`` or ``ssl_channel_credentials`` is provided.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.
            always_use_jwt_access (Optional[bool]): Whether self signed JWT should
                be used for service account credentials.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
              creation failed for any reason.
          google.api_core.exceptions.DuplicateCredentialArgs: If both ``credentials``
              and ``credentials_file`` are passed.
        """
        self._grpc_channel = None
        self._ssl_channel_credentials = ssl_channel_credentials
        self._stubs: Dict[str, Callable] = {}
        self._operations_client: Optional[operations_v1.OperationsAsyncClient] = None

        if api_mtls_endpoint:
            warnings.warn("api_mtls_endpoint is deprecated", DeprecationWarning)
        if client_cert_source:
            warnings.warn("client_cert_source is deprecated", DeprecationWarning)

        if channel:
            # Ignore credentials if a channel was passed.
            credentials = False
            # If a channel was explicitly provided, set it.
            self._grpc_channel = channel
            self._ssl_channel_credentials = None
        else:
            if api_mtls_endpoint:
                host = api_mtls_endpoint

                # Create SSL credentials with client_cert_source or application
                # default SSL credentials.
                if client_cert_source:
                    cert, key = client_cert_source()
                    self._ssl_channel_credentials = grpc.ssl_channel_credentials(
                        certificate_chain=cert, private_key=key
                    )
                else:
                    self._ssl_channel_credentials = SslCredentials().ssl_credentials

            else:
                if client_cert_source_for_mtls and not ssl_channel_credentials:
                    cert, key = client_cert_source_for_mtls()
                    self._ssl_channel_credentials = grpc.ssl_channel_credentials(
                        certificate_chain=cert, private_key=key
                    )

        # The base transport sets the host, credentials and scopes
        super().__init__(
            host=host,
            credentials=credentials,
            credentials_file=credentials_file,
            scopes=scopes,
            quota_project_id=quota_project_id,
            client_info=client_info,
            always_use_jwt_access=always_use_jwt_access,
            api_audience=api_audience,
        )

        if not self._grpc_channel:
            self._grpc_channel = type(self).create_channel(
                self._host,
                # use the credentials which are saved
                credentials=self._credentials,
                # Set ``credentials_file`` to ``None`` here as
                # the credentials that we saved earlier should be used.
                credentials_file=None,
                scopes=self._scopes,
                ssl_credentials=self._ssl_channel_credentials,
                quota_project_id=quota_project_id,
                options=[
                    ("grpc.max_send_message_length", -1),
                    ("grpc.max_receive_message_length", -1),
                ],
            )

        # Wrap messages. This must be done after self._grpc_channel exists
        self._prep_wrapped_messages(client_info)

    @property
    def grpc_channel(self) -> aio.Channel:
        """Create the channel designed to connect to this service.

        This property caches on the instance; repeated calls return
        the same channel.
        """
        # Return the channel from cache.
        return self._grpc_channel

    @property
    def operations_client(self) -> operations_v1.OperationsAsyncClient:
        """Create the client designed to process long-running operations.

        This property caches on the instance; repeated calls return the same
        client.
        """
        # Quick check: Only create a new client if we do not already have one.
        if self._operations_client is None:
            self._operations_client = operations_v1.OperationsAsyncClient(
                self.grpc_channel
            )

        # Return the client from cache.
        return self._operations_client

    @property
    def create_os_policy_assignment(
        self,
    ) -> Callable[
        [os_policy_assignments.CreateOSPolicyAssignmentRequest],
        Awaitable[operations_pb2.Operation],
    ]:
        r"""Return a callable for the create os policy assignment method over gRPC.

        Create an OS policy assignment.

        This method also creates the first revision of the OS policy
        assignment.

        This method returns a long running operation (LRO) that contains
        the rollout details. The rollout can be cancelled by cancelling
        the LRO.

        For more information, see `Method:
        projects.locations.osPolicyAssignments.operations.cancel <https://cloud.google.com/compute/docs/osconfig/rest/v1/projects.locations.osPolicyAssignments.operations/cancel>`__.

        Returns:
            Callable[[~.CreateOSPolicyAssignmentRequest],
                    Awaitable[~.Operation]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "create_os_policy_assignment" not in self._stubs:
            self._stubs["create_os_policy_assignment"] = self.grpc_channel.unary_unary(
                "/google.cloud.osconfig.v1.OsConfigZonalService/CreateOSPolicyAssignment",
                request_serializer=os_policy_assignments.CreateOSPolicyAssignmentRequest.serialize,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["create_os_policy_assignment"]

    @property
    def update_os_policy_assignment(
        self,
    ) -> Callable[
        [os_policy_assignments.UpdateOSPolicyAssignmentRequest],
        Awaitable[operations_pb2.Operation],
    ]:
        r"""Return a callable for the update os policy assignment method over gRPC.

        Update an existing OS policy assignment.

        This method creates a new revision of the OS policy assignment.

        This method returns a long running operation (LRO) that contains
        the rollout details. The rollout can be cancelled by cancelling
        the LRO.

        For more information, see `Method:
        projects.locations.osPolicyAssignments.operations.cancel <https://cloud.google.com/compute/docs/osconfig/rest/v1/projects.locations.osPolicyAssignments.operations/cancel>`__.

        Returns:
            Callable[[~.UpdateOSPolicyAssignmentRequest],
                    Awaitable[~.Operation]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "update_os_policy_assignment" not in self._stubs:
            self._stubs["update_os_policy_assignment"] = self.grpc_channel.unary_unary(
                "/google.cloud.osconfig.v1.OsConfigZonalService/UpdateOSPolicyAssignment",
                request_serializer=os_policy_assignments.UpdateOSPolicyAssignmentRequest.serialize,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["update_os_policy_assignment"]

    @property
    def get_os_policy_assignment(
        self,
    ) -> Callable[
        [os_policy_assignments.GetOSPolicyAssignmentRequest],
        Awaitable[os_policy_assignments.OSPolicyAssignment],
    ]:
        r"""Return a callable for the get os policy assignment method over gRPC.

        Retrieve an existing OS policy assignment.

        This method always returns the latest revision. In order to
        retrieve a previous revision of the assignment, also provide the
        revision ID in the ``name`` parameter.

        Returns:
            Callable[[~.GetOSPolicyAssignmentRequest],
                    Awaitable[~.OSPolicyAssignment]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_os_policy_assignment" not in self._stubs:
            self._stubs["get_os_policy_assignment"] = self.grpc_channel.unary_unary(
                "/google.cloud.osconfig.v1.OsConfigZonalService/GetOSPolicyAssignment",
                request_serializer=os_policy_assignments.GetOSPolicyAssignmentRequest.serialize,
                response_deserializer=os_policy_assignments.OSPolicyAssignment.deserialize,
            )
        return self._stubs["get_os_policy_assignment"]

    @property
    def list_os_policy_assignments(
        self,
    ) -> Callable[
        [os_policy_assignments.ListOSPolicyAssignmentsRequest],
        Awaitable[os_policy_assignments.ListOSPolicyAssignmentsResponse],
    ]:
        r"""Return a callable for the list os policy assignments method over gRPC.

        List the OS policy assignments under the parent
        resource.
        For each OS policy assignment, the latest revision is
        returned.

        Returns:
            Callable[[~.ListOSPolicyAssignmentsRequest],
                    Awaitable[~.ListOSPolicyAssignmentsResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_os_policy_assignments" not in self._stubs:
            self._stubs["list_os_policy_assignments"] = self.grpc_channel.unary_unary(
                "/google.cloud.osconfig.v1.OsConfigZonalService/ListOSPolicyAssignments",
                request_serializer=os_policy_assignments.ListOSPolicyAssignmentsRequest.serialize,
                response_deserializer=os_policy_assignments.ListOSPolicyAssignmentsResponse.deserialize,
            )
        return self._stubs["list_os_policy_assignments"]

    @property
    def list_os_policy_assignment_revisions(
        self,
    ) -> Callable[
        [os_policy_assignments.ListOSPolicyAssignmentRevisionsRequest],
        Awaitable[os_policy_assignments.ListOSPolicyAssignmentRevisionsResponse],
    ]:
        r"""Return a callable for the list os policy assignment
        revisions method over gRPC.

        List the OS policy assignment revisions for a given
        OS policy assignment.

        Returns:
            Callable[[~.ListOSPolicyAssignmentRevisionsRequest],
                    Awaitable[~.ListOSPolicyAssignmentRevisionsResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_os_policy_assignment_revisions" not in self._stubs:
            self._stubs[
                "list_os_policy_assignment_revisions"
            ] = self.grpc_channel.unary_unary(
                "/google.cloud.osconfig.v1.OsConfigZonalService/ListOSPolicyAssignmentRevisions",
                request_serializer=os_policy_assignments.ListOSPolicyAssignmentRevisionsRequest.serialize,
                response_deserializer=os_policy_assignments.ListOSPolicyAssignmentRevisionsResponse.deserialize,
            )
        return self._stubs["list_os_policy_assignment_revisions"]

    @property
    def delete_os_policy_assignment(
        self,
    ) -> Callable[
        [os_policy_assignments.DeleteOSPolicyAssignmentRequest],
        Awaitable[operations_pb2.Operation],
    ]:
        r"""Return a callable for the delete os policy assignment method over gRPC.

        Delete the OS policy assignment.

        This method creates a new revision of the OS policy assignment.

        This method returns a long running operation (LRO) that contains
        the rollout details. The rollout can be cancelled by cancelling
        the LRO.

        If the LRO completes and is not cancelled, all revisions
        associated with the OS policy assignment are deleted.

        For more information, see `Method:
        projects.locations.osPolicyAssignments.operations.cancel <https://cloud.google.com/compute/docs/osconfig/rest/v1/projects.locations.osPolicyAssignments.operations/cancel>`__.

        Returns:
            Callable[[~.DeleteOSPolicyAssignmentRequest],
                    Awaitable[~.Operation]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "delete_os_policy_assignment" not in self._stubs:
            self._stubs["delete_os_policy_assignment"] = self.grpc_channel.unary_unary(
                "/google.cloud.osconfig.v1.OsConfigZonalService/DeleteOSPolicyAssignment",
                request_serializer=os_policy_assignments.DeleteOSPolicyAssignmentRequest.serialize,
                response_deserializer=operations_pb2.Operation.FromString,
            )
        return self._stubs["delete_os_policy_assignment"]

    @property
    def get_os_policy_assignment_report(
        self,
    ) -> Callable[
        [os_policy_assignment_reports.GetOSPolicyAssignmentReportRequest],
        Awaitable[os_policy_assignment_reports.OSPolicyAssignmentReport],
    ]:
        r"""Return a callable for the get os policy assignment
        report method over gRPC.

        Get the OS policy asssignment report for the
        specified Compute Engine VM instance.

        Returns:
            Callable[[~.GetOSPolicyAssignmentReportRequest],
                    Awaitable[~.OSPolicyAssignmentReport]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_os_policy_assignment_report" not in self._stubs:
            self._stubs[
                "get_os_policy_assignment_report"
            ] = self.grpc_channel.unary_unary(
                "/google.cloud.osconfig.v1.OsConfigZonalService/GetOSPolicyAssignmentReport",
                request_serializer=os_policy_assignment_reports.GetOSPolicyAssignmentReportRequest.serialize,
                response_deserializer=os_policy_assignment_reports.OSPolicyAssignmentReport.deserialize,
            )
        return self._stubs["get_os_policy_assignment_report"]

    @property
    def list_os_policy_assignment_reports(
        self,
    ) -> Callable[
        [os_policy_assignment_reports.ListOSPolicyAssignmentReportsRequest],
        Awaitable[os_policy_assignment_reports.ListOSPolicyAssignmentReportsResponse],
    ]:
        r"""Return a callable for the list os policy assignment
        reports method over gRPC.

        List OS policy asssignment reports for all Compute
        Engine VM instances in the specified zone.

        Returns:
            Callable[[~.ListOSPolicyAssignmentReportsRequest],
                    Awaitable[~.ListOSPolicyAssignmentReportsResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_os_policy_assignment_reports" not in self._stubs:
            self._stubs[
                "list_os_policy_assignment_reports"
            ] = self.grpc_channel.unary_unary(
                "/google.cloud.osconfig.v1.OsConfigZonalService/ListOSPolicyAssignmentReports",
                request_serializer=os_policy_assignment_reports.ListOSPolicyAssignmentReportsRequest.serialize,
                response_deserializer=os_policy_assignment_reports.ListOSPolicyAssignmentReportsResponse.deserialize,
            )
        return self._stubs["list_os_policy_assignment_reports"]

    @property
    def get_inventory(
        self,
    ) -> Callable[[inventory.GetInventoryRequest], Awaitable[inventory.Inventory]]:
        r"""Return a callable for the get inventory method over gRPC.

        Get inventory data for the specified VM instance. If the VM has
        no associated inventory, the message ``NOT_FOUND`` is returned.

        Returns:
            Callable[[~.GetInventoryRequest],
                    Awaitable[~.Inventory]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_inventory" not in self._stubs:
            self._stubs["get_inventory"] = self.grpc_channel.unary_unary(
                "/google.cloud.osconfig.v1.OsConfigZonalService/GetInventory",
                request_serializer=inventory.GetInventoryRequest.serialize,
                response_deserializer=inventory.Inventory.deserialize,
            )
        return self._stubs["get_inventory"]

    @property
    def list_inventories(
        self,
    ) -> Callable[
        [inventory.ListInventoriesRequest], Awaitable[inventory.ListInventoriesResponse]
    ]:
        r"""Return a callable for the list inventories method over gRPC.

        List inventory data for all VM instances in the
        specified zone.

        Returns:
            Callable[[~.ListInventoriesRequest],
                    Awaitable[~.ListInventoriesResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_inventories" not in self._stubs:
            self._stubs["list_inventories"] = self.grpc_channel.unary_unary(
                "/google.cloud.osconfig.v1.OsConfigZonalService/ListInventories",
                request_serializer=inventory.ListInventoriesRequest.serialize,
                response_deserializer=inventory.ListInventoriesResponse.deserialize,
            )
        return self._stubs["list_inventories"]

    @property
    def get_vulnerability_report(
        self,
    ) -> Callable[
        [vulnerability.GetVulnerabilityReportRequest],
        Awaitable[vulnerability.VulnerabilityReport],
    ]:
        r"""Return a callable for the get vulnerability report method over gRPC.

        Gets the vulnerability report for the specified VM
        instance. Only VMs with inventory data have
        vulnerability reports associated with them.

        Returns:
            Callable[[~.GetVulnerabilityReportRequest],
                    Awaitable[~.VulnerabilityReport]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_vulnerability_report" not in self._stubs:
            self._stubs["get_vulnerability_report"] = self.grpc_channel.unary_unary(
                "/google.cloud.osconfig.v1.OsConfigZonalService/GetVulnerabilityReport",
                request_serializer=vulnerability.GetVulnerabilityReportRequest.serialize,
                response_deserializer=vulnerability.VulnerabilityReport.deserialize,
            )
        return self._stubs["get_vulnerability_report"]

    @property
    def list_vulnerability_reports(
        self,
    ) -> Callable[
        [vulnerability.ListVulnerabilityReportsRequest],
        Awaitable[vulnerability.ListVulnerabilityReportsResponse],
    ]:
        r"""Return a callable for the list vulnerability reports method over gRPC.

        List vulnerability reports for all VM instances in
        the specified zone.

        Returns:
            Callable[[~.ListVulnerabilityReportsRequest],
                    Awaitable[~.ListVulnerabilityReportsResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_vulnerability_reports" not in self._stubs:
            self._stubs["list_vulnerability_reports"] = self.grpc_channel.unary_unary(
                "/google.cloud.osconfig.v1.OsConfigZonalService/ListVulnerabilityReports",
                request_serializer=vulnerability.ListVulnerabilityReportsRequest.serialize,
                response_deserializer=vulnerability.ListVulnerabilityReportsResponse.deserialize,
            )
        return self._stubs["list_vulnerability_reports"]

    def close(self):
        return self.grpc_channel.close()


__all__ = ("OsConfigZonalServiceGrpcAsyncIOTransport",)
